/*
 * program-ipac.c 
 *
 *
 * Tested with 2.2.17 on the x86 arch
 */
#include "console-lib-program-ipac.inc"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/kd.h>

/* Some global variables */

#define IPAC_Y_M 0x15 /* "Y" - Acknowledge Make Code             */
#define IPAC_Y_B 0x95 /* "Y" - Acknowledge Break Code            */
#define IPAC_C_M 0x2E /* "C" - Checksum Error Make Code          */
#define IPAC_C_B 0xAE /* "C" - Checksum Error Breake Code        */
#define IPAC_J_M 0x24 /* "J" - Jumper Settings Error Make Code   */
#define IPAC_J_B 0xA4 /* "J" - Jumper Settings Error breake Code */
#define IPAC_NIL 0x00 /* Timeout Error                           */

#define LED_NO  0 /*No Leds*/
#define LED_SL  1 /*Scroll Lock*/
#define LED_NL  2 /*Num Lock*/
#define LED_CL  4 /*Caps Lock*/
#define LED_ALL 7 /*All Leds*/

unsigned char getkeyboard()
{
  unsigned char i;
  if (read(STDIN_FILENO,&i,1)){
    return i;
  } else {
    fprintf(stderr,"Didn't get any data from keyboard\n");
    return (char)0;
  };
}
    
int program_ipac(void)
{
  unsigned char s;
  unsigned char keychar;
  printf("entering function: int program_ipac(void)\n");
  tcdrain(STDIN_FILENO); // wait for all output to be transmitted
  tty_raw(STDIN_FILENO);
  tcgetattr(STDIN_FILENO,&buf); // Should not fail if tty_raw worked!
  buf.c_cc[VMIN] = 0; // bytes needed
  buf.c_cc[VTIME] = 50; //  timer in 10ths of seconds
  tcsetattr(STDIN_FILENO, TCSANOW, &buf);
  tcflush(STDIN_FILENO,TCIOFLUSH); // flush all both Input/Ouput not TCIFLUSH TCOFLUSH
  /* Program the I-Pac */

  /* 1. PC sends any let code except "all LEDS on"*/
  sendleds(LED_NL,udelay);
  
  /* 2. PC sends 5 successive "all LEDs on" codes */
  sendleds(LED_ALL,udelay);
  sendleds(LED_ALL,udelay);
  sendleds(LED_ALL,udelay);
  sendleds(LED_ALL,udelay);
  sendleds(LED_ALL,udelay);

  /* 3. I-PAC returns a character. This is as folows:
   *          "Y" means "OK, ready for code block"
   *          "J" means "Jumper set to MAME, cannot program"
   */
  keychar=getkeyboard();
  fprintf(stderr,"key read from I-Pac: %i \n",(int)keychar);
  return (0);
}

int main(int argc, char **argv)
{
  int i;
    /* First check if the program has root permissions
     * otherwise we will not be able to change the LEDs
     */
    if (geteuid() != 0){
      fprintf(stderr, "You need to run this program as root or you can not control the LEDs !\n");
      exit(2);
    } else {
      printf("Running the program with the needed permissions\n\n");
    }

    /* this sleep is here just because this is example program
     * it is needed to get the enter-key-up event you type to start the program
     * this programs start when the key-down is registered, and we humans are
     * so slow that the first key the program would read is the key-release of
     * the enterkey.
     */
    usleep(3000000); // tree seconds

    /* Let's see if we can read the current leds+keyboard state */
    keyboard_init();
    printf("initial value of leds : %i \n",saveled);

    printf("\nMake sure that no other programs are influencing the LED's!!\n\n");

    nice(-20); /* We are important :-) */

    /* program it */
    program_ipac();

    /* We now can restore the keyboard */
    restoreleds();
    tty_reset();

    /* normal exit */
    exit(0); 
}
